"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var import_promisify_child_process = require("promisify-child-process");
var import_js_controller_common = require("@iobroker/js-controller-common");
var import_semver = require("semver");
var import_js_controller_cli = require("@iobroker/js-controller-cli");
var import_node_http = __toESM(require("node:http"), 1);
var import_node_https = __toESM(require("node:https"), 1);
var import_promises = require("node:timers/promises");
var import_fs_extra = __toESM(require("fs-extra"), 1);
var import_node_url = __toESM(require("node:url"), 1);
const import_meta = {};
class UpgradeManager {
  STOP_TIMEOUT_MS = 5e3;
  SHUTDOWN_TIMEOUT = 1e4;
  adminInstance;
  version;
  response = {
    running: true,
    stderr: [],
    stdout: []
  };
  shutdownAbortController;
  logger;
  server;
  sockets = /* @__PURE__ */ new Set();
  hostname = import_js_controller_common.tools.getHostName();
  constructor(args) {
    this.adminInstance = args.adminInstance;
    this.version = args.version;
    this.logger = this.setupLogger();
  }
  setupLogger() {
    const config = import_fs_extra.default.readJSONSync(import_js_controller_common.tools.getConfigFileName());
    return (0, import_js_controller_common.logger)({ ...config.log, noStdout: false });
  }
  static parseCliCommands() {
    const additionalArgs = process.argv.slice(2);
    const version = additionalArgs[0];
    const adminInstance = parseInt(additionalArgs[1]);
    const isValid = !!(0, import_semver.valid)(version);
    if (!isValid) {
      UpgradeManager.printUsage();
      throw new Error("The provided version is not valid");
    }
    if (isNaN(adminInstance)) {
      UpgradeManager.printUsage();
      throw new Error("Please provide a valid admin instance");
    }
    return { version, adminInstance };
  }
  log(message, error = false) {
    if (error) {
      this.logger.error(`host.${this.hostname} [CONTROLLER_AUTO_UPGRADE] ${message}`);
      this.response.stderr.push(message);
      return;
    }
    this.logger.info(`host.${this.hostname} [CONTROLLER_AUTO_UPGRADE] ${message}`);
    this.response.stdout.push(message);
  }
  async stopController() {
    if (import_js_controller_common.tools.isDocker()) {
      await (0, import_promisify_child_process.exec)("/opt/scripts/maintenance.sh on -kbn");
    } else {
      await (0, import_promisify_child_process.exec)(`${import_js_controller_common.tools.appNameLowerCase} stop`);
    }
    await (0, import_promises.setTimeout)(this.STOP_TIMEOUT_MS);
  }
  startController() {
    if (import_js_controller_common.tools.isDocker()) {
      return (0, import_promisify_child_process.exec)("/opt/scripts/maintenance.sh off -y");
    }
    return (0, import_promisify_child_process.exec)(`${import_js_controller_common.tools.appNameLowerCase} start`);
  }
  static printUsage() {
    console.info('Example usage: "node upgradeManager.js <version> <adminInstance>"');
  }
  async npmInstall() {
    const res = await import_js_controller_common.tools.installNodeModule(`iobroker.js-controller@${this.version}`, {
      cwd: "/opt/iobroker",
      debug: true
    });
    this.response.stderr.push(...res.stderr.split("\n"));
    this.response.stdout.push(...res.stdout.split("\n"));
    this.response.success = res.success;
    if (!res.success) {
      throw new Error(`Could not install js-controller@${this.version}`);
    }
  }
  startWebServer(params) {
    const { useHttps } = params;
    if (useHttps) {
      this.startSecureWebServer(params);
    } else {
      this.startInsecureWebServer(params);
    }
  }
  shutdownApp() {
    if (this.shutdownAbortController) {
      this.shutdownAbortController.abort();
    }
    if (!this.server) {
      process.exit();
    }
    this.destroySockets();
    this.server.close(async () => {
      await this.startController();
      this.log("Successfully started js-controller");
      process.exit();
    });
  }
  destroySockets() {
    for (const socket of this.sockets) {
      socket.destroy();
      this.sockets.delete(socket);
    }
  }
  webServerCallback(res) {
    res.writeHead(200, {
      "Access-Control-Allow-Origin": "*"
    });
    res.end(JSON.stringify(this.response));
    if (!this.response.running) {
      this.log("Final information delivered");
      this.shutdownApp();
    }
  }
  startInsecureWebServer(params) {
    const { port } = params;
    this.server = import_node_http.default.createServer((_req, res) => {
      this.webServerCallback(res);
    });
    this.monitorSockets(this.server);
    this.server.listen(port, () => {
      this.log(`Server is running on http://localhost:${port}`);
    });
  }
  startSecureWebServer(params) {
    const { port, certPublic, certPrivate } = params;
    this.server = import_node_https.default.createServer({ key: certPrivate, cert: certPublic }, (_req, res) => {
      this.webServerCallback(res);
    });
    this.monitorSockets(this.server);
    this.server.listen(port, () => {
      this.log(`Server is running on https://localhost:${port}`);
    });
  }
  monitorSockets(server) {
    server.on("connection", (socket) => {
      this.sockets.add(socket);
      server.once("close", () => {
        this.sockets.delete(socket);
      });
    });
  }
  async getCertificates(params) {
    const { objects, certPublicName, certPrivateName } = params;
    const obj = await objects.getObjectAsync("system.certificates");
    if (!obj) {
      throw new Error("No certificates found");
    }
    const certs = obj.native.certificates;
    return { certPrivate: certs[certPrivateName], certPublic: certs[certPublicName] };
  }
  async collectWebServerParameters() {
    const { objects } = await (0, import_js_controller_cli.dbConnectAsync)(false);
    const obj = await objects.getObjectAsync(`system.adapter.admin.${this.adminInstance}`);
    if (!obj) {
      UpgradeManager.printUsage();
      throw new Error("Please provide a valid admin instance");
    }
    if (obj.native.secure) {
      const { certPublic: certPublicName, certPrivate: certPrivateName } = obj.native;
      const { certPublic, certPrivate } = await this.getCertificates({
        objects,
        certPublicName,
        certPrivateName
      });
      return {
        useHttps: obj.native.secure,
        port: obj.native.port,
        certPublic,
        certPrivate
      };
    }
    return {
      useHttps: false,
      port: obj.native.port
    };
  }
  async setFinished() {
    this.response.running = false;
    await this.startShutdownTimeout();
  }
  async startShutdownTimeout() {
    this.shutdownAbortController = new AbortController();
    try {
      await (0, import_promises.setTimeout)(this.SHUTDOWN_TIMEOUT, null, { signal: this.shutdownAbortController.signal });
      this.log("Timeout expired, initializing shutdown");
      this.shutdownApp();
    } catch (e) {
      if (e.code !== "ABORT_ERR") {
        this.log(e.message, true);
      }
    }
  }
}
async function main() {
  const upgradeArguments = UpgradeManager.parseCliCommands();
  const upgradeManager = new UpgradeManager(upgradeArguments);
  registerErrorHandlers(upgradeManager);
  const webServerParameters = await upgradeManager.collectWebServerParameters();
  upgradeManager.log("Stopping controller");
  await upgradeManager.stopController();
  upgradeManager.log("Successfully stopped js-controller");
  upgradeManager.startWebServer(webServerParameters);
  try {
    await upgradeManager.npmInstall();
  } catch (e) {
    upgradeManager.log(e.message, true);
  }
  await upgradeManager.setFinished();
}
function registerErrorHandlers(upgradeManager) {
  process.on("uncaughtException", (e) => {
    upgradeManager.log(`Uncaught Exception: ${e.stack}`, true);
  });
  process.on("unhandledRejection", (rej) => {
    upgradeManager.log(`Unhandled rejection: ${rej instanceof Error ? rej.stack : JSON.stringify(rej)}`, true);
  });
}
const modulePath = import_node_url.default.fileURLToPath(import_meta.url || "file://" + __filename);
if (process.argv[1] === modulePath) {
  main();
}
//# sourceMappingURL=upgradeManager.js.map
